@@ -48,19 +48,15 @@ aiounicast_select::aiounicast_select
 	buf_in_size = TMCG_MAX_VALUE_CHARS;
 	for (size_t i = 0; i < n_in; i++)
 	{
-		fd_in.push_back(fd_in_in[i]);
+		fd_in[i] = fd_in_in[i];
 		unsigned char *buf = new unsigned char[buf_in_size];
 		buf_in.push_back(buf), buf_ptr.push_back(0);
 		buf_flag.push_back(false);
-		fd_out.push_back(fd_out_in[i]);
+		fd_out[i] = fd_out_in[i];
 	}
 
 	// initialize ordered buffer for receiving mpz_t
-	for (size_t i = 0; i < n_in; i++)
-	{
-		std::list<mpz_ptr> *ltmp = new std::list<mpz_ptr>;
-		buf_mpz.push_back(*ltmp);
-	}
+	buf_mpz.resize(n);
 
 	// initialize character counters
 	numWrite = 0, numRead = 0, numEncrypted = 0, numDecrypted = 0, numAuthenticated = 0;
@@ -210,6 +206,9 @@ bool aiounicast_select::Send
 		return false;
 	if (timeout == aio_timeout_default)
 		timeout = aio_default_timeout;
+	// check whether output file descriptor exists
+	if (!fd_out.count(i_in))
+		return false;
 	// prepare write buffer from the message m
 	mpz_t tmp;
 	mpz_init_set(tmp, m);
@@ -316,6 +315,8 @@ bool aiounicast_select::Send
 					{
 						if ((errno == EAGAIN) || (errno == EWOULDBLOCK) || (errno == EINTR))
 						{
+							if (errno == EAGAIN)
+								perror("aiounicast_select (write)");
 							continue;
 						}
 						else
@@ -385,6 +386,8 @@ bool aiounicast_select::Send
 			{
 				if ((errno == EAGAIN) || (errno == EWOULDBLOCK) || (errno == EINTR))
 				{
+					if (errno == EAGAIN)
+						perror("aiounicast_select (write)");
 					continue;
 				}
 				else
@@ -458,6 +461,8 @@ bool aiounicast_select::Send
 				{
 					if ((errno == EAGAIN) || (errno == EWOULDBLOCK) || (errno == EINTR))
 					{
+						if (errno == EAGAIN)
+							perror("aiounicast_select (write)");
 						continue;
 					}
 					else
@@ -596,7 +601,7 @@ bool aiounicast_select::Receive
 						err = gcry_mac_verify(*mac_in[i_out], mac, maclen);
 						if (err)
 						{
-							std::cerr << "libgcrypt: gcry_mac_verify() from " << i_out << " failed" << std::endl;
+							std::cerr << "libgcrypt: gcry_mac_verify() for " << j << " from " << i_out << " failed" << std::endl;
 							std::cerr << gcry_strerror(err) << std::endl;
 							delete [] tmp, delete [] mac;
 							return false;
@@ -666,6 +671,9 @@ bool aiounicast_select::Receive
 				// no delimiter found; invalidate buffer flag
 				buf_flag[i_out] = false;
 			}
+			// check whether input file descriptor exists
+			if (!fd_in.count(i_out))
+				continue;
 			size_t maxbuf = buf_in_size - buf_ptr[i_out];
 			if (maxbuf > 0)
 			{
@@ -676,7 +684,7 @@ bool aiounicast_select::Receive
 				FD_ZERO(&rfds);
 				FD_SET(fd_in[i_out], &rfds);
 				tv.tv_sec = 0;
-				tv.tv_usec = 1000; // sleep only for 1000us = 1ms
+				tv.tv_usec = 50000; // sleep only for 50000us = 50ms
 				retval = select((fd_in[i_out] + 1), &rfds, NULL, NULL, &tv);
 				if (retval < 0)
 				{
@@ -695,6 +703,14 @@ bool aiounicast_select::Receive
 						perror("aiounicast_select (read)");
 						return false;
 					}
+					if (num == 0)
+					{
+						// got EOF
+						std::cerr << "aiounicast_select(" << j << "): got EOF for " << i_out << std::endl;
+						fd_in.erase(i_out); // erase corresponding input file descriptor
+						fd_out.erase(i_out); // erase corresponding output file descriptor
+						continue;
+					}
 					buf_ptr[i_out] += num;
 					numRead += num;
 					if (aio_is_encrypted)
