@@ -1,7 +1,8 @@
 /*******************************************************************************
    This file is part of LibTMCG.
 
- Copyright (C) 2005, 2006, 2007  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2005, 2006, 2007, 2009, 
+                     2016, 2017  Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +19,14 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/
 
+// include headers
+#ifdef HAVE_CONFIG_H
+	#include "libTMCG_config.h"
+#endif
+#include <libTMCG.hh>
+
+#ifdef FORKING
+
 #include <sstream>
 #include <vector>
 #include <cassert>
@@ -26,14 +35,13 @@
 #include <sys/wait.h>
 
 #include "test_helper.h"
-#include <libTMCG.hh>
 #include "pipestream.hh"
 
 #undef NDEBUG
 
 // create a random permutation (naive algorithm)
 void random_permutation
-	(size_t n, std::vector<size_t> &pi)
+	(const size_t n, std::vector<size_t> &pi)
 {
 	pi.clear();
 	for (size_t i = 0; i < n; i++)
@@ -59,7 +67,7 @@ void random_permutation
 
 // create a random permutation (Knuth or Fisher-Yates algorithm)
 void random_permutation_fast
-	(size_t n, std::vector<size_t> &pi)
+	(const size_t n, std::vector<size_t> &pi)
 {
 	pi.clear();
 	for (size_t i = 0; i < n; i++)
@@ -90,8 +98,9 @@ int main
 	mpz_t a, b, aa, bb;
 	assert(init_libTMCG());
 	
-	size_t pi_check_factor = 256;
-	size_t pi_check_n = 3, pi_check_size = 6 * pi_check_factor, cnt = 0;
+	const size_t pi_check_factor = 256;
+	const size_t pi_check_n = 3, pi_check_size = 6 * pi_check_factor; 
+	size_t cnt = 0;
 	std::vector<size_t> delta, alpha[pi_check_size], beta[pi_check_size];
 	for (size_t i = 0; i < pi_check_n; i++)
 		delta.push_back(i);
@@ -125,6 +134,7 @@ int main
 	mpz_init(a), mpz_init(b), mpz_init(aa), mpz_init(bb);
 	size_t n = 32;
 	
+	// check PedersenCOM
 	PedersenCommitmentScheme *com, *com2;
 	std::stringstream foo;
 	std::vector<mpz_ptr> mp;
@@ -179,7 +189,53 @@ int main
 	}
 	mp.clear();
 	delete com, delete com2;
-	
+
+	// attack PedersenCOM
+	std::stringstream bar;
+	std::cout << "attack PedersenCommitmentScheme(1)" << std::endl;
+	com = new PedersenCommitmentScheme(1);
+	std::cout << "*.CheckGroup()" << std::endl;
+	assert(com->CheckGroup());
+	mpz_wrandomm(bb, com->q);
+	mpz_powm(b, com->h, bb, com->p);
+	bar << com->p << std::endl << com->q << std::endl << com->k << std::endl << com->h << std::endl;
+	bar << b << std::endl;
+	com2 = new PedersenCommitmentScheme(1, bar);
+	std::cout << "*.CheckGroup(modified g)" << std::endl;
+	assert(com2->CheckGroup());
+	{
+		mpz_ptr tmp = new mpz_t();
+		mpz_init_set_ui(tmp, 42L);
+		mp.push_back(tmp);
+	}
+	mpz_wrandomm(aa, com->q);
+	mpz_powm(a, com2->h, aa, com2->p);
+	std::cout << "c = " << a << std::endl;
+	std::cout << "*.TestMembership(c)" << std::endl;
+	assert(com2->TestMembership(a));
+	std::cout << "*.Verify(c, r, mp)" << std::endl;
+	mpz_mul(b, bb, mp[0]);
+	mpz_mod(b, b, com2->q);
+	mpz_sub(b, aa, b);
+	mpz_mod(b, b, com2->q);
+	std::cout << "r = " << b << std::endl;
+	assert(com2->Verify(a, b, mp));
+	std::cout << "*.Verify(c, r, mp + 1)" << std::endl;
+	mpz_add_ui(mp[0], mp[0], 1L);
+	mpz_mul(b, bb, mp[0]);
+	mpz_mod(b, b, com2->q);
+	mpz_sub(b, aa, b);
+	mpz_mod(b, b, com2->q);
+	std::cout << "r = " << b << std::endl;
+	assert(com2->Verify(a, b, mp));
+	{
+		mpz_clear(mp[0]);
+		delete mp[0];
+	}
+	mp.clear();
+	delete com, delete com2;
+
+	// interactive VSSHE checks
 	pid_t pid = 0;
 	int pipe1fd[2], pipe2fd[2];
 	if ((pipe(pipe1fd) < 0) || (pipe(pipe2fd) < 0))
@@ -200,7 +256,7 @@ int main
 			std::vector<mpz_ptr> m, m_pi, R;
 			std::vector<std::pair<mpz_ptr, mpz_ptr> > e, E;
 			std::vector<size_t> pi, xi;
-			std::stringstream lej, lej2;
+			std::stringstream lej, lej2, lej3;
 			
 			mpz_init(c), mpz_init(r);
 			com->PublishGroup(*pipe_out), com->PublishGroup(lej);
@@ -255,7 +311,44 @@ int main
 			skc->Prove_interactive(pi, r, m, *pipe_in, *pipe_out);
 			stop_clock();
 			std::cout << "P: " << elapsed_time() << std::endl;
-			
+			// prove SKC wrong
+			start_clock();
+			std::cout << "P: !skc.Prove_interactive(...)" << std::endl;
+			mpz_add_ui(r, r, 1L);
+			skc->Prove_interactive(pi, r, m, *pipe_in, *pipe_out);
+			mpz_sub_ui(r, r, 1L);
+			stop_clock();
+			std::cout << "P: " << elapsed_time() << std::endl;
+			// prove SKC
+			start_clock();
+			std::cout << "P: skc.Prove_noninteractive(...)" << std::endl;
+			skc->Prove_noninteractive(pi, r, m, *pipe_out);
+			stop_clock();
+			std::cout << "P: " << elapsed_time() << std::endl;
+			// prove SKC wrong
+			start_clock();
+			std::cout << "P: !skc.Prove_noninteractive(...)" << std::endl;
+			skc->Prove_noninteractive(xi, r, m, *pipe_out);
+			stop_clock();
+			std::cout << "P: " << elapsed_time() << std::endl;
+			// prove SKC
+			start_clock();
+			std::cout << "P: skc.Prove_noninteractive(...)" << std::endl;
+			skc->Prove_noninteractive(pi, r, m, *pipe_out);
+			stop_clock();
+			std::cout << "P: " << elapsed_time() << std::endl;
+			// prove SKC wrong
+			start_clock();
+			std::cout << "P: !skc.Prove_noninteractive(...)" << std::endl;
+			mpz_add_ui(r, r, 1L);
+			skc->Prove_noninteractive(pi, r, m, *pipe_out);
+			mpz_sub_ui(r, r, 1L);
+			stop_clock();
+			std::cout << "P: " << elapsed_time() << std::endl;
+
+			// initialize EDCF
+			JareckiLysyanskayaEDCF *edcf = new JareckiLysyanskayaEDCF(2, 0,
+					com->p, com->q, com->g[0], com->h);
 			// initialize VSSHE
 			lej2 << com->p << std::endl << com->q << std::endl << com->g[0] << 
 				std::endl << com->h << std::endl;
@@ -297,6 +390,36 @@ int main
 			vsshe->Prove_interactive(xi, R, e, E, *pipe_in, *pipe_out);
 			stop_clock();
 			std::cout << "P: " << elapsed_time() << std::endl;
+			// prove VSSHE public-coin
+			start_clock();
+			std::cout << "P: vsshe.Prove_interactive_publiccoin(...)" << std::endl;
+			vsshe->Prove_interactive_publiccoin(pi, R, e, E, edcf, *pipe_in, *pipe_out);
+			stop_clock();
+			std::cout << "P: " << elapsed_time() << std::endl;
+			// prove VSSHE public-coin wrong
+			start_clock();
+			std::cout << "P: !vsshe.Prove_interactive_publiccoin(...)" << std::endl;
+			vsshe->Prove_interactive_publiccoin(xi, R, e, E, edcf, *pipe_in, *pipe_out);
+			stop_clock();
+			std::cout << "P: " << elapsed_time() << std::endl;
+			// prove VSSHE non-interactive
+			start_clock();
+			std::cout << "P: vsshe.Prove_noninteractive(...)" << std::endl;
+			vsshe->Prove_noninteractive(pi, R, e, E, *pipe_out);
+			stop_clock();
+			std::cout << "P: " << elapsed_time() << std::endl;
+			// prove VSSHE non-interactive wrong
+			start_clock();
+			std::cout << "P: !vsshe.Prove_noninteractive(...)" << std::endl;
+			vsshe->Prove_noninteractive(xi, R, e, E, *pipe_out);
+			stop_clock();
+			std::cout << "P: " << elapsed_time() << std::endl;
+
+			// NSHVZKA
+			vsshe->Prove_noninteractive(pi, R, e, E, lej3);
+			std::string NSHVZKA = lej3.str();
+			std::cout << "NSHVZKA(" << NSHVZKA.size() << " bytes)" << std::endl;
+			assert(vsshe->Verify_noninteractive(e, E, lej3));
 			
 			// release
 			for (size_t i = 0; i < n; i++)
@@ -310,7 +433,7 @@ int main
 			}
 			m.clear(), m_pi.clear(), R.clear(), e.clear(), E.clear();
 			mpz_clear(c), mpz_clear(r);
-			delete vsshe, delete skc, delete com;
+			delete vsshe, delete edcf, delete skc, delete com;
 			
 			delete pipe_in, delete pipe_out;
 			exit(0);
@@ -319,6 +442,7 @@ int main
 		else
 		{
 			std::cout << "fork() = " << pid << std::endl;
+
 			/* Verifier */
 			ipipestream *pipe_in = new ipipestream(pipe2fd[0]);
 			opipestream *pipe_out = new opipestream(pipe1fd[1]);
@@ -348,8 +472,8 @@ int main
 			std::cout << "V: c = " << a << std::endl;
 			// verify SKC
 			start_clock();
-			std::cout << "V: skc.Verify_interactive(...)" << std::endl;
-			assert(skc->Verify_interactive(a, m, *pipe_in, *pipe_out));
+			std::cout << "V: skc.Verify_interactive(..., false)" << std::endl;
+			assert(skc->Verify_interactive(a, m, *pipe_in, *pipe_out, false));
 			stop_clock();
 			std::cout << "V: " << elapsed_time() << std::endl;
 			// verify SKC wrong
@@ -364,7 +488,40 @@ int main
 			assert(skc->Verify_interactive(a, m, *pipe_in, *pipe_out));
 			stop_clock();
 			std::cout << "V: " << elapsed_time() << std::endl;
-			
+			// verify SKC wrong
+			start_clock();
+			std::cout << "V: !skc.Verify_interactive(...)" << std::endl;
+			assert(!skc->Verify_interactive(a, m, *pipe_in, *pipe_out));
+			stop_clock();
+			std::cout << "V: " << elapsed_time() << std::endl;
+			// verify SKC
+			start_clock();
+			std::cout << "V: skc.Verify_noninteractive(..., false)" << std::endl;
+			assert(skc->Verify_noninteractive(a, m, *pipe_in, false));
+			stop_clock();
+			std::cout << "V: " << elapsed_time() << std::endl;
+			// verify SKC wrong
+			start_clock();
+			std::cout << "V: !skc.Verify_noninteractive(...)" << std::endl;
+			assert(!skc->Verify_noninteractive(a, m, *pipe_in));
+			stop_clock();
+			std::cout << "V: " << elapsed_time() << std::endl;
+			// verify SKC
+			start_clock();
+			std::cout << "V: skc.Verify_noninteractive(...)" << std::endl;
+			assert(skc->Verify_noninteractive(a, m, *pipe_in));
+			stop_clock();
+			std::cout << "V: " << elapsed_time() << std::endl;
+			// verify SKC wrong
+			start_clock();
+			std::cout << "V: !skc.Verify_noninteractive(...)" << std::endl;
+			assert(!skc->Verify_noninteractive(a, m, *pipe_in));
+			stop_clock();
+			std::cout << "V: " << elapsed_time() << std::endl;
+
+			// initialize EDCF
+			JareckiLysyanskayaEDCF *edcf = new JareckiLysyanskayaEDCF(2, 0,
+					com->p, com->q, com->g[0], com->h);	
 			// initialize VSSHE
 			lej2 << com->p << std::endl << com->q << std::endl << com->g[0] << 
 				std::endl << com->h << std::endl;
@@ -374,18 +531,42 @@ int main
 			for (size_t i = 0; i < n; i++)
 				*pipe_in >> e[i].first >> e[i].second >> E[i].first >> 
 					E[i].second;
-			// prove VSSHE
+			// verify VSSHE
 			start_clock();
 			std::cout << "V: vsshe.Verify_interactive(...)" << std::endl;
 			assert(vsshe->Verify_interactive(e, E, *pipe_in, *pipe_out));
 			stop_clock();
 			std::cout << "V: " << elapsed_time() << std::endl;
-			// prove VSSHE wrong
+			// verify VSSHE wrong
 			start_clock();
 			std::cout << "V: !vsshe.Verify_interactive(...)" << std::endl;
 			assert(!vsshe->Verify_interactive(e, E, *pipe_in, *pipe_out));
 			stop_clock();
 			std::cout << "V: " << elapsed_time() << std::endl;
+			// verify VSSHE public-coin
+			start_clock();
+			std::cout << "V: vsshe.Verify_interactive_publiccoin(...)" << std::endl;
+			assert(vsshe->Verify_interactive_publiccoin(e, E, edcf, *pipe_in, *pipe_out));
+			stop_clock();
+			std::cout << "V: " << elapsed_time() << std::endl;
+			// verify VSSHE public-coin wrong
+			start_clock();
+			std::cout << "V: !vsshe.Verify_interactive_publiccoin(...)" << std::endl;
+			assert(!vsshe->Verify_interactive_publiccoin(e, E, edcf, *pipe_in, *pipe_out));
+			stop_clock();
+			std::cout << "V: " << elapsed_time() << std::endl;
+			// verify VSSHE non-interactive
+			start_clock();
+			std::cout << "V: vsshe.Verify_noninteractive(...)" << std::endl;
+			assert(vsshe->Verify_noninteractive(e, E, *pipe_in));
+			stop_clock();
+			std::cout << "V: " << elapsed_time() << std::endl;
+			// verify VSSHE non-interactive wrong
+			start_clock();
+			std::cout << "V: !vsshe.Verify_noninteractive(...)" << std::endl;
+			assert(!vsshe->Verify_noninteractive(e, E, *pipe_in));
+			stop_clock();
+			std::cout << "V: " << elapsed_time() << std::endl;
 			
 			// release
 			for (size_t i = 0; i < n; i++)
@@ -398,7 +579,7 @@ int main
 				delete E[i].first, delete E[i].second;
 			}
 			m.clear(), e.clear(), E.clear();
-			delete vsshe, delete skc, delete com;
+			delete vsshe, delete edcf, delete skc, delete com;
 			
 			delete pipe_in, delete pipe_out;
 		}
@@ -410,3 +591,14 @@ int main
 	mpz_clear(a), mpz_clear(b), mpz_clear(aa), mpz_clear(bb);
 	return 0;
 }
+
+#else
+
+int main
+	(int argc, char **argv)
+{
+	std::cout << "test skipped" << std::endl;
+	return 77;
+}
+
+#endif
